"{ Package: 'stx:goodies/monticello' }"

MCFileBasedRepository subclass:#MCHttpRepository
	instanceVariableNames:'location user password readerCache'
	classVariableNames:''
	poolDictionaries:''
	category:'Monticello-Repositories'
!


!MCHttpRepository class methodsFor:'as yet unclassified'!

creationTemplate
	^self creationTemplateLocation: 'http://foo.com/bar'
		user: 'squeak'
		password: 'squeak'
!

creationTemplateLocation: location user: user password: password
	^
'MCHttpRepository
	location: {1}
	user: {2}
	password: {3}' format: {location printString. user printString. password printString}
!

description
	^ 'HTTP'
!

fillInTheBlankRequest
	^ 'HTTP Repository:'
			
!

location: location user: user password: password
	^ self new
		location: location;
		user: user;
		password: password
!

morphicConfigure
	^ self fillInTheBlankConfigure
! !

!MCHttpRepository class methodsFor:'class initialization'!

clearPasswords
	self allSubInstancesDo: [:ea | ea password: ''].
! !

!MCHttpRepository methodsFor:'as yet unclassified'!

asCreationTemplate
	^self class creationTemplateLocation: location user: user password: password
!

flushCache
	super flushCache.
	readerCache _ nil
!

location: aUrlString
	location _ aUrlString
!

locationWithTrailingSlash
	^ (location endsWith: '/')
		ifTrue: [location]
		ifFalse: [location, '/']
!

parseFileNamesFromStream: aStream
	| names fullName |
	names _ OrderedCollection new.
	[aStream atEnd] whileFalse:
		[[aStream upTo: $<. {$a. $A. nil} includes: aStream next] whileFalse.
		aStream upTo: $".
		aStream atEnd ifFalse: [
			fullName _ aStream upTo: $".
			names add: fullName unescapePercents]].
	^ names
!

password
	self userAndPasswordFromSettingsDo: [:usr :pwd | ^pwd].

	self user isEmpty ifTrue: [^password].

	[password isEmpty] whileTrue: [
		| answer |
		"Give the user a chance to change the login"
		answer _ FillInTheBlank request: 'User name for ', String cr, location
			initialAnswer: self user.
		answer isEmpty
			ifTrue: [^password]
			ifFalse: [self user: answer].
		
		password := FillInTheBlank requestPassword: 'Password for "', self user, '" at ', String cr, location.
	].

	^ password
!

password: passwordString
	password _ passwordString
!

urlForFileNamed: aString
	^ self locationWithTrailingSlash, aString encodeForHTTP
!

user
	self userAndPasswordFromSettingsDo: [:usr :pwd | ^usr].
	"not in settings"
	^user
!

user: userString
	user _ userString
!

userAndPasswordFromSettingsDo: aBlock
	"The mcSettings file in ExternalSettings preferenceDirectory should contain entries for each account:
	
		account1: *myhost.mydomain* user:password
		account2: *otherhost.mydomain/somerep* dXNlcjpwYXNzd29yZA==

	That is it must start with 'account', followed by anything to distinguish accounts, and a colon. Then comes a match expression for the repository url, and after a space the user:password string.
	
	To not have the clear text password on your disc, you can base64 encode it:
			(Base64MimeConverter mimeEncode: 'user:password' readStream) contents
	"

	| entry userAndPassword |
	Settings ifNotNil: [
		Settings keysAndValuesDo: [:key :value |
			(key asLowercase beginsWith: 'account') ifTrue: [
				entry _ value findTokens: '	 '.
				(entry first match: location) ifTrue: [
					userAndPassword := entry second.
					(userAndPassword includes: $:) ifFalse: [
						userAndPassword := (Base64MimeConverter mimeDecodeToChars: userAndPassword readStream) contents].
					userAndPassword := userAndPassword findTokens: $:.
					^aBlock value: userAndPassword first 
						value: userAndPassword second 
					]
			]
		]
	].
	^nil
!

versionReaderForFileNamed: aString
	readerCache ifNil: [readerCache _ Dictionary new].
	^ readerCache at: aString ifAbsent:
		[self resizeCache: readerCache.
		super versionReaderForFileNamed: aString do:
			[:r |
			r ifNotNil: [readerCache at: aString put: r]]]
	
!

versionReaderForFileNamed: aString do: aBlock
	^ (self versionReaderForFileNamed: aString) ifNotNilDo: aBlock
! !

!MCHttpRepository methodsFor:'required'!

allFileNames
	| index |
	index _ HTTPSocket httpGet: self locationWithTrailingSlash, '?C=M;O=D' args: nil user: self user passwd: self password.
	index isString ifTrue: [self error: 'Could not access ', location].
	^ self parseFileNamesFromStream: index	
!

description
	^ location
!

readStreamForFileNamed: aString do: aBlock
	| contents |
	contents _ HTTPSocket httpGet: (self urlForFileNamed: aString) args: nil user: self user passwd: self password.
	^ contents isString ifFalse: [aBlock value: contents]
!

writeStreamForFileNamed: aString replace: ignoreBoolean do: aBlock
	| stream response |
	stream _ RWBinaryOrTextStream on: String new.
	aBlock value: stream.
	response _ HTTPSocket
					httpPut: stream contents
					to: (self urlForFileNamed: aString)
					user: self user
					passwd: self password.

	(#( 'HTTP/1.1 201 ' 'HTTP/1.1 200 ' 'HTTP/1.0 201 ' 'HTTP/1.0 200 ')
		anySatisfy: [:code | response beginsWith: code ])
			ifFalse: [self error: response].
! !

!MCHttpRepository class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/monticello/MCHttpRepository.st,v 1.1 2006-11-22 13:22:38 cg Exp $'
! !
